#ifndef _PPC_BYTEORDER_H
#define _PPC_BYTEORDER_H

#ifndef __BIG_ENDIAN
#define __BIG_ENDIAN 4321
#endif

#ifndef __BIG_ENDIAN_BITFIELD
#define __BIG_ENDIAN_BITFIELD
#endif

#if 0 /* Assume PowerPC is Big-Endian! */
#undef ntohl
#undef ntohs
#undef htonl
#undef htons

extern unsigned long int	ntohl(unsigned long int);
extern unsigned short int	ntohs(unsigned short int);
extern unsigned long int	htonl(unsigned long int);
extern unsigned short int	htons(unsigned short int);

extern unsigned long int	__ntohl(unsigned long int);
extern unsigned short int	__ntohs(unsigned short int);
extern unsigned long int	__constant_ntohl(unsigned long int);
extern unsigned short int	__constant_ntohs(unsigned short int);

extern __inline__ unsigned long int
__ntohl(unsigned long int x)
{
	return (((x & 0x000000ffU) << 24) |
		((x & 0x0000ff00U) <<  8) |
		((x & 0x00ff0000U) >>  8) |
		((x & 0xff000000U) >> 24));
}

extern __inline__ unsigned long int
__constant_ntohl(unsigned long int x)
{
	return (((x & 0x000000ffU) << 24) |
		((x & 0x0000ff00U) <<  8) |
		((x & 0x00ff0000U) >>  8) |
		((x & 0xff000000U) >> 24));
}

extern __inline__ unsigned short int
__ntohs(unsigned short int x)
{
	return (((x & 0x00ff) << 8) |
		((x & 0xff00) >> 8));
}

extern __inline__ unsigned short int
__constant_ntohs(unsigned short int x)
{
	return (((x & 0x00ff) << 8) |
		((x & 0xff00) >> 8));
}

#define __htonl(x) __ntohl(x)
#define __htons(x) __ntohs(x)
#define __constant_htonl(x) __constant_ntohl(x)
#define __constant_htons(x) __constant_ntohs(x)

#ifdef  __OPTIMIZE__
#  define ntohl(x) \
(__builtin_constant_p((long)(x)) ? \
 __constant_ntohl((x)) : \
 __ntohl((x)))
#  define ntohs(x) \
(__builtin_constant_p((short)(x)) ? \
 __constant_ntohs((x)) : \
 __ntohs((x)))
#  define htonl(x) \
(__builtin_constant_p((long)(x)) ? \
 __constant_htonl((x)) : \
 __htonl((x)))
#  define htons(x) \
(__builtin_constant_p((short)(x)) ? \
 __constant_htons((x)) : \
 __htons((x)))
#endif

#else

#define ntohl(x) ((unsigned long int) (x))
#define ntohs(x) ((unsigned short int) (x))
#define htonl(x) ((unsigned long int) (x))
#define htons(x) ((unsigned short int) (x))

#if 0 /* OLD */
/*
 * In-kernel byte order macros to handle stuff like
 * byte-order-dependent filesystems etc.
 */
#define cpu_to_le32(x) le32_to_cpu((x))
extern __inline__ unsigned long le32_to_cpu(unsigned long x)
{
     	return (((x & 0x000000ffU) << 24) |
		((x & 0x0000ff00U) <<  8) |
		((x & 0x00ff0000U) >>  8) |
		((x & 0xff000000U) >> 24));
}


#define cpu_to_le16(x) le16_to_cpu((x))
extern __inline__ unsigned short le16_to_cpu(unsigned short x)
{
	return (((x & 0x00ff) << 8) |
		((x & 0xff00) >> 8));
}

#define cpu_to_be32(x) (x)
#define be32_to_cpu(x) (x)
#define cpu_to_be16(x) (x)
#define be16_to_cpu(x) (x)
#endif
#endif

/*
 * Adapted from 2.1.XX kernel mods
 */

#ifdef __KERNEL__
/*
 * 16 and 32 bit little-endian loads and stores.
 */
extern inline unsigned ld_le16(volatile unsigned short *addr)
{
	unsigned val;

	asm volatile("lhbrx %0,0,%1" : "=r" (val) : "r" (addr));
	return val;
}

extern inline void st_le16(volatile unsigned short *addr, unsigned val)
{
	asm volatile("sthbrx %0,0,%1" : : "r" (val), "r" (addr) : "memory");
}

extern inline unsigned ld_le32(volatile unsigned long *addr)
{
	unsigned val;

	asm volatile("lwbrx %0,0,%1" : "=r" (val) : "r" (addr));
	return val;
}

extern inline void st_le32(volatile unsigned long *addr, unsigned val)
{
	asm volatile("stwbrx %0,0,%1" : : "r" (val), "r" (addr) : "memory");
}

#if 0
extern __inline__ unsigned short cpu_to_le16(unsigned short value)
{
	return ld_le16(&value);
}
extern __inline__ unsigned long cpu_to_le32(unsigned long value)
{
	return ld_le32(&value);
}
#else
extern __inline__ unsigned short cpu_to_le16(unsigned short value)
{
	unsigned short result;

	asm("rlwimi %0,%1,8,16,23"
	    : "=r" (result)
	    : "r" (value), "0" (value >> 8));
	return result;
}
extern __inline__ unsigned long cpu_to_le32(unsigned long value)
{
	unsigned long result;

	asm("rlwimi %0,%1,24,16,23\n\t"
	    "rlwimi %0,%1,8,8,15\n\t"
	    "rlwimi %0,%1,24,0,7"
	    : "=r" (result)
	    : "r" (value), "0" (value >> 24));
	return result;
}
#endif /* 0 */

#define cpu_to_be16(x)  (x)
#define cpu_to_be32(x)  (x)

/* The same, but returns converted value from the location pointer by addr. */
extern __inline__ unsigned short cpu_to_le16p(unsigned short *addr)
{
	return ld_le16(addr);
}

extern __inline__ unsigned long cpu_to_le32p(unsigned long *addr)
{
	return ld_le32(addr);
}

extern __inline__ unsigned short cpu_to_be16p(unsigned short *addr)
{
	return *addr;
}

extern __inline__ unsigned long cpu_to_be32p(unsigned long *addr)
{
	return *addr;
}

/* The same, but do the conversion in situ, ie. put the value back to addr. */
extern __inline__ void cpu_to_le16s(unsigned short *addr)
{
	st_le16(addr,*addr);
}

extern __inline__ void cpu_to_le32s(unsigned long *addr)
{
	st_le32(addr,*addr);
}

#define cpu_to_be16s(x) do { } while (0)
#define cpu_to_be32s(x) do { } while (0)

/* Convert from specified byte order, to CPU byte order. */
#define le16_to_cpu(x)  cpu_to_le16(x)
#define le32_to_cpu(x)  cpu_to_le32(x)
#define be16_to_cpu(x)  cpu_to_be16(x)
#define be32_to_cpu(x)  cpu_to_be32(x)

#define le16_to_cpup(x) cpu_to_le16p(x)
#define le32_to_cpup(x) cpu_to_le32p(x)
#define be16_to_cpup(x) cpu_to_be16p(x)
#define be32_to_cpup(x) cpu_to_be32p(x)

#define le16_to_cpus(x) cpu_to_le16s(x)
#define le32_to_cpus(x) cpu_to_le32s(x)
#define be16_to_cpus(x) cpu_to_be16s(x)
#define be32_to_cpus(x) cpu_to_be32s(x)


#endif /* __KERNEL__ */

#endif /* !(_PPC_BYTEORDER_H) */
